CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  EN34_PIN      = 21
  IN4_PIN       = 20
  IN3_PIN       = 19
  EN12_PIN      = 18 ' beware of cables
  IN2_PIN       = 17
  IN1_PIN       = 16
  DELAY_PER_STEP= 50_000
  PWM_PERIOD    = 10000
  FWD           = 70
  MAX_POWER     = 70

  MAX_TICKS_PER_S = 32

  OUT           = 1

  ODOM_R_PIN    = 12
  ODOM_L_PIN    = 13
  IN = 0
  WHEEL_DEBOUNCE  = 20

  SPEED_SAMPLING_INTERVAL = 20_000_000

  ' wheel has 24 ticks per revolution
  ' radius is 3cm 
  ' so circunference = 10 cm aprox
  ' 10cm / s -> 24 ticks/s
  ' sample every 250 ms 
 
OBJ
  pwmL : "pwmAsm"
  pwmR : "pwmAsm"

VAR
  long motorStack[80]
  ' wheel watcher vars
  long stateL, stateR, last_changeR, last_changeL
  'tick ctrs
  long wheelL_ctr, wheelR_ctr
  ' signal sent to hbridge
  long powerL, powerR
  ' desired speed
  long targetR, targetL
  long last_speed_sampling
  long speedL, speedR
  long speedL_ptr, speedR_ptr
  long powerR_ptr, powerL_ptr

PUB start(speedL_ptr_, speedR_ptr_, powerL_ptr_, powerR_ptr_)
  powerL_ptr := powerL_ptr_
  powerR_ptr := powerR_ptr_
  speedR_ptr := speedR_ptr_
  speedL_ptr := speedL_ptr_

  ' init motors
  pwmL.start(EN12_PIN)
  pwmR.start(EN34_PIN)
  pwmL.setPeriod(PWM_PERIOD)
  pwmR.setPeriod(PWM_PERIOD)

  ' init vars
  long[speedR_ptr] := 0
  long[speedL_ptr] := 0
  wheelR_ctr := 0
  wheelL_ctr := 0
  speedL     := 0
  speedR     := 0
  targetL    := 0
  targetR    := 0

  cognew(motor_loop, @motorStack)

PUB set_speed(targetL_, targetR_)
  ' targets are in (-100,100) range
  ' we take it to ticks/sec range, where
  ' 100 -> MAX_TICKS_PER_S ticks/s
  '-100 ->-MAX_TICKS_PER_S ticks/s
  targetL := targetL_ 
  targetR := targetR_ 

PRI motor_loop
  dira[ODOM_R_PIN] := IN
  dira[ODOM_L_PIN] := IN
  dira[IN1_PIN] := OUT
  dira[IN2_PIN] := OUT
  dira[IN3_PIN] := OUT                                    
  dira[IN4_PIN] := OUT

  stateR := ina[ODOM_R_PIN]
  stateL := ina[ODOM_L_PIN] 

  last_changeR := last_changeL := cnt - WHEEL_DEBOUNCE

  last_speed_sampling := cnt 

  repeat
    watch_wheels 
    sample_speed
    long[speedL_ptr] := speedL
    long[speedR_ptr] := speedR
    long[powerL_ptr] := powerL
    long[powerR_ptr] := powerR
    
    adjust_power
    ' waitcnt(cnt + DELAY_PER_STEP)

PRI watch_wheels 
   
  if ina[ODOM_L_PIN] <> stateL      
    if (cnt - last_changeL) > WHEEL_DEBOUNCE
      last_changeL := cnt
      wheelL_ctr++
      stateL := ina[ODOM_L_PIN]
  else
    last_changeL #>= (cnt - WHEEL_DEBOUNCE)

  if ina[ODOM_R_PIN] <> stateR      
    if (cnt - last_changeR) > WHEEL_DEBOUNCE
      last_changeR := cnt
      wheelR_ctr++
      stateR := ina[ODOM_R_PIN]
  else
    last_changeR #>= (cnt - WHEEL_DEBOUNCE)
 

PRI sample_speed
  if (cnt - last_speed_sampling) > SPEED_SAMPLING_INTERVAL   
    ' convert to % of max speed
    ' 4 factor is for 1/4 sampling rate
    speedL := (4*100*wheelL_ctr)/MAX_TICKS_PER_S
    speedR := (4*100*wheelR_ctr)/MAX_TICKS_PER_S
    wheelL_ctr := wheelR_ctr := 0
    last_speed_sampling := cnt

PRI adjust_power 
  if targetR > speedR and powerR < MAX_POWER 
    powerR++          
  elseif targetR < speedR and powerR > 0
    powerR--
        
  if targetL > speedL and powerL < MAX_POWER
    powerL++          
  elseif targetL < speedL and powerL > 0    
    powerL--

  set_hbridge(powerL, powerR)


PRI set_hbridge(sgn_left, sgn_right) | in1, in2, in3, in4 
  ' input: signed speed/power
  ' but L298 uses direction + pwm
  ' in[1-4] determines direction, pwm determines power
  if sgn_right > 0
    in1 := 1
    in2 := 0
  elseif sgn_right < 0
    in1 := 0 
    in2 := 1 
  else
    in1 := in2 := 0

  if sgn_left > 0
    in3 := 1
    in4 := 0
  elseif sgn_left < 0
    in3 := 0 
    in4 := 1 
  else
    in3 := in4 := 0

  ' keep absolute value of input for pwm
  ||sgn_right
  ||sgn_left 

  outa[IN1_PIN] := in1        
  outa[IN2_PIN] := in2
  outa[IN3_PIN] := in3
  outa[IN4_PIN] := in4
  pwmL.setDuty(sgn_left)
  pwmR.setDuty(sgn_right)

