CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  PING_L_PIN    = 10
  PING_M_PIN    = 9
  PING_R_PIN    = 11

  CRUISE_TIME   = 180
  NAV_REV_TIME  = 500
  NAV_TURN_AROUND_TIME = 700

  NAV_HARD_TURN_TIME = 300 'was 500

  TOO_CLOSE_THRESH = 28 
  TOO_TOO_CLOSE_THRESH = 10

OBJ
  clk   : "clock"
  ping  : "Ping"
  omot  : "open_motor"

VAR 
  long stackNAV[80]

  long pings_ptr
  long motorL_ptr
  long motorR_ptr
  long beh_ctrl_ptr
  long messages_ptr

  long rangeL, rangeR, rangeM

  long msg_straight
  long msg_rev
  long msg_R
  long msg_L
  long msg_disabled

PUB start(pings_ptr_, beh_ctrl_ptr_, motorL_ptr_, motorR_ptr_, messages_ptr_)
  pings_ptr    := pings_ptr_
  motorL_ptr   := motorL_ptr_
  motorR_ptr   := motorR_ptr_
  beh_ctrl_ptr := beh_ctrl_ptr_
  messages_ptr := messages_ptr_

  msg_straight := string("straight         ")
  msg_L        := string("left             ")
  msg_R        := string("right            ")
  msg_disabled := string("disabled nav     ")
  msg_rev      := string("reversed         ")

  long[messages_ptr] := msg_disabled

  clk.init(_xinfreq)

  cognew(navigate, @stackNAV)

PRI navigate 
  rangeL := rangeR := rangeM := 0

  repeat    
    rangeL += ping.centimeters(PING_L_PIN)
    rangeL /=2
    clk.pausemsec(30)
    rangeR += ping.centimeters(PING_R_PIN)
    rangeR /=2
    clk.pausemsec(30)
    rangeM += ping.centimeters(PING_M_PIN)
    rangeM /=2
    clk.pausemsec(30)

    long[pings_ptr][0] := rangeL
    long[pings_ptr][1] := rangeM
    long[pings_ptr][2] := rangeR


    if (rangeM < TOO_TOO_CLOSE_THRESH) or (rangeR < TOO_TOO_CLOSE_THRESH) or (rangeR < TOO_TOO_CLOSE_THRESH)
      short_stop
      reverse
      turn_around
    elseif (rangeM < TOO_CLOSE_THRESH)
      ' messages[NAV_BEH] := string("NAV: too close front ")
      if (rangeR > rangeL)
        hard_right
      else
        hard_left
    elseif (rangeL < TOO_CLOSE_THRESH) 
      hard_right
    elseif (rangeR < TOO_CLOSE_THRESH) 
      hard_left
    else
      cruise

PRI cruise | diff, absdiff 
  long[beh_ctrl_ptr] := 1            
  long[messages_ptr] := msg_straight
  long[motorL_ptr] := long[motorR_ptr] := omot#SPEED
  clk.pausemsec(100)
  'if rangeR - rangeL > 50
  '  absdiff := ||diff
  '  long[motorL_ptr] := omot#SPEED + 3 
  '  long[motorR_ptr] := omot#SPEED + absdiff) <# 80

  diff := rangeL - rangeR ' max should be around 300
  absdiff := ||diff
  absdiff <#= 30 ' absdiff = min(absdiff,30)
  if absdiff > 20        
    if (diff > 0) ' L > R -> more speed to right motor
      'messages[NAV_BEH] := string("NAV: cruise left     ")
      long[motorL_ptr] := (omot#SPEED - absdiff) #> 55 
      long[motorR_ptr] := (omot#SPEED + absdiff) <# 80
    else
      'messages[NAV_BEH] := string("NAV: cruise right    ")         
      long[motorL_ptr] := (omot#SPEED + absdiff) <# 80
      long[motorR_ptr] := (omot#SPEED - absdiff) #> 55
  clk.pausemsec(100)  
                                               
   

PRI reverse
  long[messages_ptr] := msg_rev 
  long[beh_ctrl_ptr] := 1            
  long[motorL_ptr] := long[motorR_ptr] := -omot#SPEED
  clk.pausemsec(NAV_REV_TIME)
  short_stop

PRI turn_around 
  long[messages_ptr] := msg_rev 
  long[beh_ctrl_ptr] := 1            
  long[motorL_ptr] := omot#SPEED
  long[motorR_ptr] := -omot#SPEED
  clk.pausemsec(NAV_TURN_AROUND_TIME)            
  short_stop

PRI hard_right
  long[messages_ptr] := msg_R
  long[beh_ctrl_ptr] := 1            
  long[motorL_ptr] := omot#SPEED
  long[motorR_ptr] := -omot#SPEED
  clk.pausemsec(NAV_HARD_TURN_TIME)            
  short_stop

PRI hard_left
  long[messages_ptr] := msg_L
  long[beh_ctrl_ptr] := 1            
  long[motorL_ptr] := -omot#SPEED
  long[motorR_ptr] := omot#SPEED
  clk.pausemsec(NAV_HARD_TURN_TIME)            
  short_stop

PRI short_stop
  long[beh_ctrl_ptr] := 1            
  long[motorL_ptr] := long[motorR_ptr] := 0
  ' clk.pausemsec(20)

PRI disable
  long[messages_ptr] := msg_disabled
  long[beh_ctrl_ptr] := 0
  long[motorL_ptr] := long[motorR_ptr] := 0
 
'--------------------------------------;
'NAV: L: 000 M: 000 R: 000 
'NAV: MESSAGE
'--------------------------------------;
