CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
                   
  RLED_PIN      = 23
  ' YLED_PIN      = 24
  LCD_TX_PIN    = 15

  ' misc defs =====
  OUT           = 1
  IN            = 0
  PORT_A        = 0
  CR            = $0D

  ' settings
  LCD_ENABLED   = true
  WW_ENABLED    = false
  IR_ENABLED    = true
  MOTORS_ENABLED= true
  TXT_ENABLED   = false
  NAV_ENABLED   = true
  ESC_ENABLED   = true
  ATTACK_ENABLED= true

  ' motor connection:
  ' right motor (connected on back): r/b b/r
  ' left  motor : r/r b/b white/white

  STUCK_THRESH  = 20

  ' behaviors
  MAX_BEH = 3
  #0, ATTACK_BEH, ESCAPE_BEH, NAV_BEH
 
OBJ
  clk   : "Clock"
  lcd   : "Debug_Lcd"
  txt   : "PC_Text"
  nav   : "navigator"
  ww    : "wheelwatcher"
  irdet : "irdetect"
  omotor: "open_motor"
  escape: "escape"
  attack: "attack"
  num   : "simple_numbers"
  
VAR
  byte pressedL, pressedR
  long wheelR, wheelL

  long ir_detect_ctr[irdet#MAX_IR_SENSORS]
  long ir_detect_bool[irdet#MAX_IR_SENSORS]
  long ir_detect_state

  long pings[3]
  long messages[MAX_BEH]

  long beh_ctrl[MAX_BEH]

  long motor_left[MAX_BEH]
  long motor_right[MAX_BEH]

  long spaces

  long active_beh

  long stuck_accumulator

  byte stuck

  

PUB main | i, random_dir
  init
  lcd.home
  lcd.str(string("bootup OK"))
  led_blips(RLED_PIN)

  random_dir := 28
  
  repeat
    active_beh := NAV_BEH
    repeat i from 0 to MAX_BEH - 1
      if beh_ctrl[i] == 1
        active_beh := i
        quit

    if MOTORS_ENABLED
      if ((||motor_left[active_beh] + ||motor_right[active_beh]) > 2) and (wheelR + wheelL == 0)
        if stuck_accumulator < STUCK_THRESH 
          stuck_accumulator++
          stuck := false
        else
          stuck := true
          
      if stuck
        random_dir?
        if random_dir > 0
          omotor.set_speed(-70,-70)
          clk.pausemsec(500)
        else
          omotor.set_speed(70, 70)
          clk.pausemsec(500)
        random_dir?
        if random_dir > 0
          omotor.set_speed(-70,70)
          clk.pausemsec(500)
        else
          omotor.set_speed(70,-70)
          clk.pausemsec(500)
        stuck := false
        stuck_accumulator := 0        
        
      else
        omotor.set_speed(motor_left[active_beh], motor_right[active_beh])    
        stuck_accumulator := 0
          
    if TXT_ENABLED 
      txt_refresh

    if LCD_ENABLED
      lcd_refresh

    !outa[RLED_PIN]
    clk.pausemsec(10)    
      
PRI init

  stuck_accumulator := 0
  stuck := false

  spaces := string("    ")

  longfill(@beh_ctrl, 0, MAX_BEH)
  longfill(@motor_left, 0, MAX_BEH)
  longfill(@motor_right, 0, MAX_BEH)

  clk.init(_xinfreq)

  dira[RLED_PIN] := OUT

  if LCD_ENABLED  
    lcd_init    

  if TXT_ENABLED
    txt.start(42)  

  if IR_ENABLED
    irdet.start(@ir_detect_ctr)

  if ATTACK_ENABLED 
    attack.start(@ir_detect_state, @ir_detect_ctr, @ir_detect_bool, @beh_ctrl[ATTACK_BEH], @motor_left[ATTACK_BEH], @motor_right[ATTACK_BEH], @messages[ATTACK_BEH])

  if ESC_ENABLED
    escape.start(@pressedL, @pressedR, @wheelL, @wheelR, @beh_ctrl[ESCAPE_BEH], @motor_left[ESCAPE_BEH], @motor_right[ESCAPE_BEH], @messages[ESCAPE_BEH])
    
  if WW_ENABLED
     ww.start(@wheelL, @wheelR)
     
  if NAV_ENABLED
    nav.start(@pings, @beh_ctrl[NAV_BEH], @motor_left[NAV_BEH], @motor_right[NAV_BEH], @messages[NAV_BEH])

  if MOTORS_ENABLED
    omotor.start

PRI led_blips(pin)  
  repeat 5
    outa[pin] := 1
    clk.pausemsec(20)
    outa[pin] := 0
    clk.pausemsec(20)
    
PRI lcd_refresh | i, current_line, diff
  lcd.home
  current_line := 0

  lcd.str(string("Active: "))

  case active_beh
    ATTACK_BEH: 
      lcd.str(string("ATK="))
    ESCAPE_BEH: 
      lcd.str(string("ESC="))
    NAV_BEH:
      lcd.str(string("NAV="))

  lcd.decf(motor_right[active_beh], 3)
  lcd.putc(",")
  lcd.decf(motor_left[active_beh], 3)
  ' lcd.gotoxy(0,(++current_line)//4)

  {if NAV_ENABLED  
    lcd.str(string("NA|"))
    repeat i from 0 to 2
      lcd.decf(pings[2 - i], 3)
      lcd.putc(",")
    lcd.str(string("   "))
    lcd.gotoxy(0,(++current_line)//4)
    lcd.str(messages[NAV_BEH])
    lcd.gotoxy(0,(++current_line)//4)
    }

  if IR_ENABLED
    lcd.str(string("IR|"))
    lcd.gotoxy(0, 1)
    repeat i from 0 to irdet#MAX_IR_SENSORS - 1
      ' lcd.decx(ir_detect_ctr[irdet#MAX_IR_SENSORS - i + 1],2)
      lcd.decx(ir_detect_ctr[i],2)
      lcd.putc(" ")
    'lcd.gotoxy(0, (++current_line)//4)

  if ATTACK_ENABLED    
    lcd.gotoxy(0, 2)
    lcd.str(string("AT| "))
    lcd.str(messages[ATTACK_BEH])
    lcd.gotoxy(0, 3)
    lcd.str(string("diff"))
    diff := ir_detect_ctr[irdet#IRFR_PIN] - ir_detect_ctr[irdet#IRFL_PIN]
    lcd.decf(diff, 3)
    'lcd.gotoxy(0, (++current_line)//4)

PRI lcd_init  
  lcd.init(LCD_TX_PIN,19200,4)
  lcd.backlight(true)
  lcd.cls
  clk.pausemsec(10)
  lcd.cls
  clk.pausemsec(10)
  lcd.str(string("lcd OK"))
  clk.pausemsec(10)

PRI txt_refresh | i, str_ptr
'--------------------------------------;
  txt.home
  txt.setcolor(0)

  txt.setcolor(6)
  txt.str(string("Active: "))
  case active_beh
    ATTACK_BEH: 
      txt.str(string("Attack        "))
    ESCAPE_BEH: 
      txt.str(string("Escape        "))
    NAV_BEH:
      txt.str(string("Navigate      "))
  txt.cr

  if MOTORS_ENABLED
    txt.setcolor(5)
    txt.str(string("MOT| "))
    str_ptr := num.decf(motor_left[active_beh], 3)
    txt.str(str_ptr)
    txt.out(",")
    str_ptr := num.decf(motor_right[active_beh], 3)
    txt.str(str_ptr)
    txt.out("|")
    txt.dec(stuck_accumulator)
    txt.out("|")
    if stuck
      txt.str(string(" stuck!"))
    else
      txt.str(string("       "))
    txt.cr

  if WW_ENABLED  
    txt.setcolor(0)
    txt.str(string("WW | "))
    str_ptr := num.decf(wheelL, 3)
    txt.str(str_ptr)
    txt.out(",")
    str_ptr := num.decf(wheelR, 3)
    txt.str(str_ptr)
    txt.cr

  if NAV_ENABLED
    txt.setcolor(1)
    txt.str(string("NAV| "))
    str_ptr := num.decf(pings[0], 3)
    txt.str(str_ptr)
    txt.out(",")
    str_ptr := num.decf(pings[1], 3)
    txt.str(str_ptr)
    txt.out(",")
    str_ptr := num.decf(pings[2], 3)
    txt.str(str_ptr)
    txt.cr
    txt.str(messages[NAV_BEH])
    txt.cr

  if ESC_ENABLED
    txt.setcolor(3)
    txt.str(string("ESC| "))
    txt.hex(pressedL,2)
    txt.out(",")
    txt.hex(pressedR,2)
    txt.out("|")
    str_ptr := num.decf(wheelL, 3)
    txt.str(str_ptr)
    txt.out(",")
    str_ptr := num.decf(wheelR, 3)
    txt.str(str_ptr)
    ' txt.str(spaces)
    txt.cr
    txt.str(messages[ESCAPE_BEH])
    txt.cr

  if IR_ENABLED
    txt.setcolor(2)
    txt.str(string("IR | "))
    repeat i from 0 to irdet#MAX_IR_SENSORS - 1    
      str_ptr := num.decf(ir_detect_ctr[i], 3)
      txt.str(str_ptr)
      if i <> irdet#MAX_IR_SENSORS - 1
        txt.out(",")
    txt.cr

  if ATTACK_ENABLED
    txt.setcolor(4)
    txt.str(string("ATK| ir: "))
    case ir_detect_state
      attack#IR_LOCK:
        txt.str(string("lock       "))
      attack#IR_RIGHT_FRONT:
        txt.str(string("right front"))
      attack#IR_LEFT_FRONT:
        txt.str(string("left front "))
      attack#IR_RIGHT:
        txt.str(string("right      "))
      attack#IR_LEFT:
        txt.str(string("left       "))
      attack#IR_NOTHING:
        txt.str(string("nothing    "))
    txt.cr
    txt.str(messages[ATTACK_BEH])
    txt.cr
